'use strict';

var jssha = require('jssha');

// JWTs are encoded with base64url
var base64 = null;
if (typeof btoa !== 'undefined') {
  base64 = btoa;
} else if (typeof Buffer !== 'undefined') {
  base64 = function base64(s) {
    return Buffer.from(s, 'binary').toString('base64');
  };
}
var base64ToBase64URL = function base64ToBase64URL(s) {
  return s.replace(/=/g, "").replace(/\+/g, "-").replace(/\//g, "_");
};

var base64URL = function base64URL(s) {
  return base64ToBase64URL(base64(s));
};

// Only implement HS256 for now
var sign = (function (_ref) {
  var secret = _ref.secret,
      data = _ref.data,
      _ref$iss = _ref.iss,
      iss = _ref$iss === undefined ? 'smartforms' : _ref$iss,
      _ref$iat = _ref.iat,
      iat = _ref$iat === undefined ? new Date().getTime() / 1000 : _ref$iat,
      nbf = _ref.nbf,
      exp = _ref.exp;

  var header = JSON.stringify({
    typ: 'JWT',
    alg: 'HS256'
  });
  var payload = JSON.stringify({
    iss: iss,
    iat: Math.ceil(iat),
    nbf: Math.ceil(nbf),
    exp: Math.ceil(exp),
    payload: data
  });

  var sigInput = base64URL(header) + '.' + base64URL(payload);

  var hasher = new jssha('SHA-256', 'TEXT');
  hasher.setHMACKey(secret, 'TEXT');
  hasher.update(sigInput);

  var sig = base64ToBase64URL(hasher.getHMAC('B64'));

  return sigInput + '.' + sig;
});

var viewModes = Object.freeze({
  Form: 0,
  List: 1,
  FormTest: 7,
  Admin: 8
});

var createRequest = function createRequest(_ref) {
  var clientId = _ref.clientId,
      origin = _ref.origin,
      initiatorId = _ref.initiatorId,
      secret = _ref.secret,
      _ref$viewMode = _ref.viewMode,
      viewMode = _ref$viewMode === undefined ? viewModes.Form : _ref$viewMode,
      _ref$userId = _ref.userId,
      userId = _ref$userId === undefined ? null : _ref$userId,
      _ref$admin = _ref.admin,
      admin = _ref$admin === undefined ? false : _ref$admin,
      _ref$roles = _ref.roles,
      roles = _ref$roles === undefined ? [] : _ref$roles,
      _ref$portalData = _ref.portalData,
      portalData = _ref$portalData === undefined ? {} : _ref$portalData,
      _ref$sessionData = _ref.sessionData,
      sessionData = _ref$sessionData === undefined ? {} : _ref$sessionData,
      _ref$requestData = _ref.requestData,
      requestData = _ref$requestData === undefined ? {} : _ref$requestData,
      _ref$expiresSeconds = _ref.expiresSeconds,
      expiresSeconds = _ref$expiresSeconds === undefined ? 300 : _ref$expiresSeconds,
      _ref$notBeforeSeconds = _ref.notBeforeSeconds,
      notBeforeSeconds = _ref$notBeforeSeconds === undefined ? -300 : _ref$notBeforeSeconds;

  var firstEntry = {
    client_id: clientId,
    origin: origin,
    view_options: {
      initiator_id: initiatorId,
      view_mode: viewMode,
      application_code: null
    }
  };
  var userDataPayload = JSON.stringify({
    user_id: userId || 'anonymous-' + Math.ceil(Math.random() * new Date().getTime()),
    admin: admin,
    roles: roles,
    data: null,
    portal_data: portalData,
    session_data: sessionData,
    request_data: requestData
  });
  var now = new Date().getTime() / 1000;
  var token = sign({
    secret: secret,
    data: userDataPayload,
    nbf: now + notBeforeSeconds,
    exp: now + expiresSeconds
  });
  firstEntry.user_data = token;
  return firstEntry;
};

var index = {
  createRequest: createRequest,
  ViewMode: viewModes
};

module.exports = index;
