import sign from './jwt-hs256'

const viewModes = Object.freeze({
  Form: 0,
  List: 1,
  FormTest: 7,
  Admin: 8
})

const createRequest = 
({ clientId, origin, initiatorId, secret, viewMode = viewModes.Form, 
  userId = null, admin = false, roles = [], portalData = {}, sessionData = {}, requestData = {},
  expiresSeconds = 300, notBeforeSeconds = -300 }) => {
  let firstEntry = {
    client_id: clientId,
    origin,
    view_options: {
      initiator_id: initiatorId,
      view_mode: viewMode,
      application_code: null
    }
  }
  const userDataPayload = JSON.stringify({
    user_id: userId || 'anonymous-' + Math.ceil(Math.random() * new Date().getTime()),
    admin,
    roles,
    data: null,
    portal_data: portalData,
    session_data: sessionData,
    request_data: requestData  
  })
  const now = new Date().getTime() / 1000
  const token = sign({
    secret,
    data: userDataPayload, 
    nbf: now + notBeforeSeconds,
    exp: now + expiresSeconds
  })
  firstEntry.user_data = token
  return firstEntry
}

export default {
  createRequest,
  ViewMode: viewModes
}