const jssha = require('jssha')

// JWTs are encoded with base64url
let base64 = null
if (typeof btoa !== 'undefined') {
  base64 = btoa
} else if (typeof Buffer !== 'undefined') {
  base64 = (s) => Buffer.from(s, 'binary').toString('base64') 
}
const base64ToBase64URL = (s) => (s)
  .replace(/=/g, "")
  .replace(/\+/g, "-")
  .replace(/\//g, "_")

const base64URL = (s) => base64ToBase64URL(base64(s))

// Only implement HS256 for now
export default ({ secret, data, iss = 'smartforms', iat = new Date().getTime() / 1000, nbf, exp}) => {
  const header = JSON.stringify({
    typ: 'JWT',
    alg: 'HS256'
  })
  const payload = JSON.stringify({
    iss,
    iat: Math.ceil(iat),
    nbf: Math.ceil(nbf),
    exp: Math.ceil(exp),
    payload: data,
  })

  const sigInput =  `${base64URL(header)}.${base64URL(payload)}`

  const hasher = new jssha('SHA-256', 'TEXT')
  hasher.setHMACKey(secret, 'TEXT')
  hasher.update(sigInput)
  
  const sig = base64ToBase64URL(hasher.getHMAC('B64'))

  return `${sigInput}.${sig}`
}