import pkg from './package.json'
import closure from 'rollup-plugin-closure-compiler'
import babel from 'rollup-plugin-babel'
import resolve from 'rollup-plugin-node-resolve'
import VuePlugin from 'rollup-plugin-vue'

export default [
  // browser-friendly UMD build
  {
    input: 'lib/browser/exported.js',
    output: {
      name: 'smartformsInit',
      file: pkg.browser,
      format: 'iife'
    },
    plugins: [
      resolve(), closure()
    ]
  },
  {
    input: 'lib/api/index.js',
    output: {
      name: 'api',
      file: 'api.js',
      format: 'cjs'
    },
    plugins: [
      resolve(),
      babel({
        presets: [['env', { modules: false }]]
      })
    ]
  },
  {
    input: 'lib/vue/SmartForm.vue',
    output: {
      name: 'smartforms',
      file: 'vue.js',
      format: 'cjs'
    },
    plugins: [
      resolve(), 
      VuePlugin({
        css: false
      }),
      babel({
        presets: [['env', { modules: false, targets: { uglify: true }}]]
      })
    ]
  },
  {
    external: ['react', 'react-dom'],
    input: 'lib/react/SmartForm.js',
    output: {
      name: 'smartforms',
      file: 'react.js',
      format: 'cjs',
      globals: {
        'react': 'React',
        'react-dom': 'ReactDOM'
      }  
    },
    plugins: [
      resolve(), 
      babel({
        presets: ['react', ['env', { modules: false }]]
      })
    ]
  }
];
